Package: CoSIA
Type: Package
Title: An Investigation Across Different Species and Tissues
Version: 1.13.1
Authors@R: 
  c(person(given = "Anisha",
           family = "Haldar",
           role = "aut",
           comment = c(ORCID = "0000-0002-1395-9793")),
    person(given = c("Vishal", "H."),
           family = "Oza",
           role = "aut",
           comment = c(ORCID = "0000-0001-6990-0267")),
    person(given = c("Amanda", "D."),
           family = "Clark",
           role = c("aut"),
           email = "clarkaduab@gmail.com",
           comment = c(ORCID = "0000-0002-1186-3114")),
    person(given = c("Anthony", "B."),
           family = "Crumley",
           role = c("cre", "aut"),
           email = "acrumley@uab.edu",
           comment = c(ORCID = "0009-0004-7347-4722")),
    person(given = c("Nathaniel", "S."),
           family = "DeVoss",
           role = "aut",
           comment = c(ORCID = "0000-0003-0465-2770")),
    person(given = c("Brittany", "N."),
           family = "Lasseigne",
           role = c("aut"),
           comment = c(ORCID = "0000-0002-1642-8904")))
Description: 
  Cross-Species Investigation and Analysis (CoSIA) is a package that provides
  researchers with an alternative methodology for comparing across species and
  tissues using normal wild-type RNA-Seq Gene Expression data from Bgee. Using
  RNA-Seq Gene Expression data, CoSIA provides multiple visualization tools to
  explore the transcriptome diversity and variation across genes, tissues, and
  species. CoSIA uses the Coefficient of Variation and Shannon Entropy and
  Specificity to calculate transcriptome diversity and variation. CoSIA also
  provides additional conversion tools and utilities to provide a streamlined
  methodology for cross-species comparison.
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.3.0), methods (>= 4.3.0), ExperimentHub (>= 2.7.0)
Imports: dplyr (>= 1.0.7), magrittr (>= 2.0.1), RColorBrewer (>=
        1.1-2), tidyr (>= 1.2.0), plotly (>= 4.10.0), stringr (>=
        1.4.0), ggplot2 (>= 3.3.5), tibble (>= 3.1.7), org.Hs.eg.db (>=
        3.12.0), org.Mm.eg.db (>= 3.12.0), org.Dr.eg.db (>= 3.12.0),
        org.Ce.eg.db (>= 3.12.0), org.Dm.eg.db (>= 3.12.0),
        org.Rn.eg.db (>= 3.12.0), AnnotationDbi (>= 1.52.0), biomaRt
        (>= 2.46.3), homologene (>= 1.4.68.19), annotationTools (>=
        1.64.0), readr (>= 2.1.1), tidyselect (>= 1.1.2), stats (>=
        4.1.2)
RoxygenNote: 7.3.3
Suggests: BiocStyle (>= 2.22.0), tidyverse (>= 1.3.1), knitr (>= 1.42),
        rmarkdown (>= 2.20), testthat (>= 3.1.6), qpdf (>= 1.3.0)
biocViews: Software, BiologicalQuestion, GeneExpression,
        MultipleComparison, ThirdPartyClient, DataImport, GUI
BugReports: https://github.com/lasseignelab/CoSIA/issues
URL: https://www.lasseigne.org/
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/CoSIA
git_branch: devel
git_last_commit: 090a204
git_last_commit_date: 2026-05-27
Repository: Bioconductor 3.24
Date/Publication: 2026-06-01
NeedsCompilation: no
Packaged: 2026-06-01 22:04:19 UTC; biocbuild
Author: Anisha Haldar [aut] (ORCID: <https://orcid.org/0000-0002-1395-9793>),
  Vishal H. Oza [aut] (ORCID: <https://orcid.org/0000-0001-6990-0267>),
  Amanda D. Clark [aut] (ORCID: <https://orcid.org/0000-0002-1186-3114>),
  Anthony B. Crumley [cre, aut] (ORCID:
    <https://orcid.org/0009-0004-7347-4722>),
  Nathaniel S. DeVoss [aut] (ORCID:
    <https://orcid.org/0000-0003-0465-2770>),
  Brittany N. Lasseigne [aut] (ORCID:
    <https://orcid.org/0000-0002-1642-8904>)
Maintainer: Anthony B. Crumley <acrumley@uab.edu>
