#Global Variables
utils::globalVariables(c("Anatomical_entity_name", "Anatomical_entity_ID", 
                         "Species", "Frequency", "Ensembl_ID","VST",".",
                         "Scaled_Median_VST", "mad", "Specificity",
                         "Diversity", "is"))
